#include <stdio.h>
#include <assert.h>
#include "ovt.h"

int can_not_check_beyond = 0;

/* No elem beyond ->used should be constructed. Check this using our
   shadow elem state array. */
void check_beyond_used(ovt_t *a)
{
	if (can_not_check_beyond)
		return;
	int n;
	for(n = a->used; n < sizeof(a->elem_state); n++)
		assert(a->elem_state[n] == 0);
}

/* Print the array and also check for elem state consistency between 
   the in-elem cookie and the shadow elem state array */
void print_arr(const char *header, ovt_t *a)
{
	int n;
	const char *st;

	printf("%s [u=%d a=%d]", header, a->used, a->alloced);
	for(n = 0, st = a->elem_state; n < a->used; n++) {
		const char *state;
		switch(a->array[n].cookie) {
			case cookie_alloced:  state = "*"; if (!can_not_check_beyond) assert(*st == 1); break;
			case cookie_deadslot: state = "x"; if (!can_not_check_beyond) assert(*st == 0); break;
			default:              state = "?";
		}
		printf(" %s(%s)", state, a->array[n].str);
	}
	printf("\n");
	check_beyond_used(a);
}

/* Set up an array with constructor/desctructor */
#define auto_init(arr) \
do { \
	ovt_init(&arr); \
	arr.elem_constructor = obj_constructor; \
	arr.elem_destructor  = obj_destructor; \
	arr.elem_copy        = obj_copy; \
	memset(arr.elem_state, 0, sizeof(arr.elem_state)); \
} while(0)

/* initialize a static (off-array) object using sprintf; implies construction */
#define oprintf(dst, ...) \
do { \
	int __len__; \
	__len__ = sprintf((dst)->str, __VA_ARGS__); \
	(dst)->str[__len__] = '\0'; \
	(dst)->cookie = cookie_alloced; \
} while(0)

/* Uninit an array and make sure no constructed item is left */
#define auto_uninit(arr) \
do { \
	ovt_uninit(arr); \
	check_beyond_used(arr); \
} while(0);
